function [ out , funct ] = zoom_TV(in, K, nbIter)
//
//    function [ out , funct ] = zoom_TV(in, K, nbIter)
//
// zoome "in", d'un rapport "K", en minimisant la variation totale sous la contrainte 
// d'echantillonnage a "in"
//
//

global EPSILON 
EPSILON = 0.1;

[sx,sy] = size(in);
sKx=sx*K;
sKy=sy*K;

// Initialisation
out = echantillonne(in, K, 1);

for iter = 1:nbIter
	descente = moins_gradient(out);
	step= armijo_step(out,descente);
	out = projection(out + step * descente,in);
	funct(iter) = feval(out);

	printf('iteration %-5d,      step %-10g,       functional value %-10g\n',iter,step,funct(iter));
end

endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function proj = projection(out,in)
	K=size(out,1)/size(in,1);
	tmp = in - echantillonne(out, K, 0);
	proj=out + echantillonne(tmp, K, 1);


endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function descente = moins_gradient(out)
	global EPSILON 
	Dx = grad_x(out,0);
	Dy = grad_y(out,0);
	norme = sqrt(Dx.*Dx+Dy.*Dy+EPSILON);
	descente = - ( grad_x(Dx./norme,1) + grad_y(Dy./norme,1) ) ;


endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function func = feval(out)
	global EPSILON 
	Dx = grad_x(out,0);
	Dy = grad_y(out,0);
	func=sum(sum( sqrt(Dx.*Dx+Dy.*Dy+EPSILON) ));


endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function  [stepi, fi] = armijo_step(out,descente)
    beta=0.5; 
    sigma=0.5; 
    kmax=50;
    stepi=EPSILON;
    
    norm_descente = - sum(sum(descente.*descente));
	f0= feval(out);
	fi=feval(out+stepi*descente);
	k=0;
	while (fi > f0+sigma*stepi*norm_descente) & (k<kmax)
		k=k+1;
		stepi=beta*stepi;
		fi=feval(out+stepi*descente);
	end
	
 

endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function out = grad_x(in,adjoint)

[sx,sy]=size(in);

if adjoint == 0
	tmp = [in(2:sx,:) ; in(1,:)];
	out = tmp - in;
else
	tmp = [ in(sx,:) ; in(1:sx-1,:) ];
	out = tmp - in;
end


endfunction;

  
//////////////////////////////////////////////////////////////////////////////
function out = grad_y(in,adjoint)

[sx,sy]=size(in);

if adjoint == 0
	tmp = [in(:,2:sy) , in(:,1)];
	out = tmp - in;
else
	tmp = [ in(:,sy) , in(:,1:sy-1) ];
	out = tmp - in;
end


endfunction;

  
//////////////////////////////////////////////////////////////////////////////

 



